@@ -78,7 +78,7 @@ class Users(DataModel, DB.Base, NotifyTable):
                 else:
                     self.__struct[tag] = UserProperties(tag, v, self.__user)
                     DB.session.add(self.__struct[tag])
-                self.__dict[name] = v
+                self.__dict[name] = self.__struct[tag].val
                 return
             if v is None:
                 v = []
@@ -130,7 +130,6 @@ class Users(DataModel, DB.Base, NotifyTable):
 
     ID = Column("id", INTEGER(10, unsigned=True), nullable=False, primary_key=True, unique=True)
     username = Column("username", VARCHAR(320, charset="ascii"), nullable=False, unique=True)
-    altname = OptionalC(111, "NULL", Column("altname", VARCHAR(64), unique=True))
     _password = Column("password", VARCHAR(40), nullable=False, server_default="")
     domainID = Column("domain_id", INTEGER(10, unsigned=True), nullable=False, index=True)
     maildir = Column("maildir", VARCHAR(128), nullable=False, server_default="")
@@ -152,6 +151,8 @@ class Users(DataModel, DB.Base, NotifyTable):
                                order_by="UserProperties.orderID", back_populates="user")
     aliases = relationship("Aliases", cascade="all, delete-orphan", single_parent=True, passive_deletes=True,
                            back_populates="main")
+    altnames = relationship("Altnames", cascade="all, delete-orphan", single_parent=True, passive_deletes=True,
+                            back_populates="user")
     fetchmail = OptionalNC(75, [],
                            relationship("Fetchmail", cascade="all, delete-orphan", single_parent=True,
                                         order_by="Fetchmail.active.desc()", back_populates="user"))
@@ -163,8 +164,7 @@ class Users(DataModel, DB.Base, NotifyTable):
     _dictmapping_ = ((Id(), Text("username", flags="patch")),
                      (Id("domainID", flags="init"),
                       {"attr": "ldapID", "flags": "patch"},
-                      Int("status", flags="patch"),
-                      Text("altname", flags="patch")),
+                      Int("status", flags="patch")),
                      (Text("lang", match=False, flags="patch"),
                       BoolP("pop3_imap", flags="patch"),
                       BoolP("smtp", flags="patch"),
@@ -175,6 +175,7 @@ class Users(DataModel, DB.Base, NotifyTable):
                       BoolP("privFiles", flags="patch"),
                       BoolP("privArchive", flags="patch"),
                       RefProp("aliases", flags="patch, managed", link="aliasname", flat="aliasname", qopt=selectinload),
+                      RefProp("altnames", flags="patch, managed", link="altname", qopt=selectinload),
                       RefProp("fetchmail", flags="managed, patch", link="ID", qopt=selectinload),
                       {"attr": "properties", "flags": "patch", "func": lambda p: p.namemap()},
                       RefProp("roles", qopt=selectinload),
@@ -214,7 +215,6 @@ class Users(DataModel, DB.Base, NotifyTable):
 
     NORMAL = 0
     SUSPENDED = 1
-    OUTOFDATE = 2
     DELETED = 3
     SHARED = 4
     CONTACT = 5
@@ -268,7 +268,7 @@ class Users(DataModel, DB.Base, NotifyTable):
         self.fromdict(props, *args, **kwargs)
         self.addressStatus = (self.addressStatus or 0) | status
 
-    def fromdict(self, patches, syncStore=False, *args, **kwargs):
+    def fromdict(self, patches, syncStore=True, *args, **kwargs):
         isContact = patches.get("status", self.status) != Users.CONTACT
         if "username" in patches and patches["username"] != self.username:
             from orm.domains import Domains
@@ -285,7 +285,7 @@ class Users(DataModel, DB.Base, NotifyTable):
                 self.username = username
             if not isContact and not formats.email.match(self.username):
                 raise ValueError("'{}' is not a valid e-mail address".format(self.username))
-        DataModel.fromdict(self, patches, args, kwargs)
+        DataModel.fromdict(self, patches, *args, **kwargs)
         displaytype = self.properties.get("displaytypeex", 0)
         if displaytype in (0, 1, 7, 8):
             self._deprecated_addressType, self._deprecated_subType = self._decodeDisplayType(displaytype)
@@ -293,7 +293,15 @@ class Users(DataModel, DB.Base, NotifyTable):
             with Service("chat", errors=Service.SUPPRESS_INOP) as chat:
                 self._chatUser = chat.updateUser(self, False)
         if syncStore == "always" or (syncStore and "properties" in patches):
-            self.syncStore()
+            delete = [PropTags.deriveTag(tag) for tag, val in patches["properties"].items() if val is None] \
+                if "properties" in patches else None
+            self.syncStore(delete=delete)
+
+    def todict(self, spec, *args, **kwargs):
+        data = DataModel.todict(self, spec, *args, **kwargs)
+        if isinstance(spec, int) and spec >= 2:
+            self.embedStoreProperties()
+        return data
 
     @staticmethod
     def _decodeDisplayType(displaytype):
@@ -490,6 +498,9 @@ class Users(DataModel, DB.Base, NotifyTable):
 
     @status.setter
     def status(self, val):
+        from tools.license import getLicense
+        if self.status and not val and Users.count() >= getLicense().users:
+            raise ValueError("License user limit exceeded")
         self.addressStatus = ((self.addressStatus or 0) & ~self.USER_MASK) | (val & self.USER_MASK)
 
     @status.expression
@@ -709,20 +720,19 @@ class Users(DataModel, DB.Base, NotifyTable):
 
     @validates("username")
     def validateUsername(self, key, value, *args):
-        if not self.username == value and Users.query.filter(Users.altname == value).count():
+        if not self.username == value and Altnames.query.filter(Altnames.altname == value).count():
             raise ValueError("Username is already used as alternative name")
         return value
 
-    @validates("altname")
-    def validateAltname(self, key, value, *args):
-        if Users.query.filter(Users.username == value).count():
-            raise ValueError("Alternative name is already in use")
-        return value or None
-
-    def syncStore(self):
+    def syncStore(self, delete=None):
         """Write all properties to the exmdb store.
 
         For users without a store, this function has no effect.
+
+        Parameters
+        ----------
+        delete : List[int], optional
+            List of tags to delete. The default is None.
         """
         if not self.maildir:
             return
@@ -735,6 +745,23 @@ class Users(DataModel, DB.Base, NotifyTable):
                     pass
             client = exmdb.user(self)
             client.setStoreProperties(0, props)
+            if delete:
+                client.removeStoreProperties(delete)
+
+    def embedStoreProperties(self):
+        """Retrieve store properties and embed them in MySQL properties.
+
+        If the user has no store, the call has no effect.
+        """
+        if not self.maildir:
+            return
+        with Service("exmdb", errors=Service.SUPPRESS_ALL) as exmdb:
+            client = exmdb.user(self)
+            tags = client.getAllStoreProperties()
+            tags = [t for t in tags if t & 0xFFFF not in (PropTypes.BINARY, PropTypes.BINARY_ARRAY)]
+            props = client.getStoreProperties(0, tags)
+            self.properties.update({prop.tag: prop.val for prop in props})
+            DB.session.commit()
 
 
 class UserProperties(DB.Base):
@@ -832,6 +859,22 @@ class Aliases(DataModel, DB.Base, NotifyTable):
             sysd.reloadService("gromox-delivery.service", "gromox-http.service", "gromox-zcore.service")
 
 
+class Altnames(DataModel, DB.Base):
+    __tablename__ = "altnames"
+
+    userID = Column("user_id", INTEGER(unsigned=True), ForeignKey(Users.ID, ondelete="cascade", onupdate="cascade"), primary_key=True)
+    altname = Column("altname", VARCHAR(320), primary_key=True)
+    magic = Column("magic", INTEGER(unsigned=True), server_default="0")
+
+    user = relationship(Users, back_populates="altnames")
+
+    def __init__(self, data, user, *args, **kwargs):
+        self.user = user
+        self.fromdict(data)
+
+    _dictmapping_ = ((Text("altname", flags="patch"), Int("magic"),),)
+
+
 class Fetchmail(DataModel, DB.Base):
     __tablename__ = "fetchmail"
 
