@@ -84,7 +84,9 @@ def _defaultConfig():
             "dashboard": {
                 "services": []
                 },
-            "serverPolicy": "round-robin"
+            "serverPolicy": "round-robin",
+            "updateLogPath": "/var/log/grommunio-update.log",
+            "updateSkriptPath": "/usr/sbin/grommunio-update",
             },
         "security": {
             "jwtPrivateKeyFile": "/var/db/grommunio-admin-api/auth-private.pem",
@@ -104,7 +106,9 @@ def _defaultConfig():
         "chat": {
             "connection": {},
             },
-        "tasq": {},
+        "tasq": {
+            "stackTraceDepth": 7,
+            },
         }
 
 
