@@ -133,6 +133,10 @@ class ExmdbCodes(_ReverseLookup):
 
 
 class PropTags(_ReverseLookup):
+    """
+        These tags are also accessed via getattr() calls and thus effectively
+        exposed to AWEB.
+    """
     ABPROVIDERID = 0x36150102
     ACCESS = 0x0FF40003
     ACCESSCONTROLLISTDATA = 0x3FE00102
@@ -1045,18 +1049,21 @@ class PropTags(_ReverseLookup):
             from datetime import datetime
             from .rop import ntTime
             import time
-            if not isinstance(value, datetime):
-                try:
-                    value = datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
-                except TypeError:
-                    raise ValueError("Invalid date '{}'".format(value))
-            value = ntTime(time.mktime(value.timetuple()))
+            if not isinstance(value, int):
+                if not isinstance(value, datetime):
+                    try:
+                        value = datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
+                    except TypeError:
+                        raise ValueError("Invalid date '{}'".format(value))
+                value = ntTime(time.mktime(value.timetuple()))
         if type(value) != PropTypes.pyType(baseType):
             try:
                 value = PropTypes.pyType(baseType)(value)
             except Exception:
                 raise ValueError("Type of value {} does not match type of tag {} ({})"
                                  .format(value, cls.lookup(tag), PropTypes.lookup(tag)))
+        if tag == cls.DISPLAYTYPEEX:
+            value = value & ~0x40000000  # Remove DTE_FLAG_ACL_CAPABLE flag because it is not recognized properly by gromox
         return value
 
     @classmethod
