@@ -84,7 +84,8 @@ class Worker:
                 duration = time()-start
             except Exception as err:
                 import traceback
-                self.log("ERROR", traceback.format_exc(5))
+                from .config import Config
+                self.log("ERROR", traceback.format_exc(Config["tasq"]["stackTraceDepth"]))
                 task.state = Task.ERROR
                 task.message = type(err).__name__+": "+" - ".join(str(arg) for arg in err.args)[:160]
         task.state = max(task.state, Task.COMPLETED)
@@ -145,7 +146,7 @@ class Worker:
         result, code = importObject(candidate, ldap, lang=lang)
         if code == 201:
             return dict(ID=result["ID"], username=result["username"], code=201, message="User created")
-        return dict(username=candidate.email, code=code, message=result)
+        return dict(username=candidate.email, code=code, message=result["message"])
 
     def _ldapSyncImportContact(self, candidate, ldap, orgID, domains):
         from tools.ldap import importContact
